@@ -3,7 +3,6 @@
  * This file is part of GtkSourceView
  *
  * Copyright (C) 2009 - Jesse van den Kieboom
- * Copyright (C) 2013 - Sébastien Wilmet
  *
  * gtksourceview is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,11 +23,13 @@
 
 #include <string.h>
 
+#define GTK_SOURCE_COMPLETION_WORDS_LIBRARY_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_SOURCE_TYPE_COMPLETION_WORDS_LIBRARY, GtkSourceCompletionWordsLibraryPrivate))
+
 enum
 {
 	LOCK,
 	UNLOCK,
-	N_SIGNALS
+	NUM_SIGNALS
 };
 
 struct _GtkSourceCompletionWordsLibraryPrivate
@@ -37,9 +38,9 @@ struct _GtkSourceCompletionWordsLibraryPrivate
 	gboolean locked;
 };
 
-static guint signals[N_SIGNALS];
+static guint signals[NUM_SIGNALS] = {0,};
 
-G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceCompletionWordsLibrary, gtk_source_completion_words_library, G_TYPE_OBJECT)
+G_DEFINE_TYPE (GtkSourceCompletionWordsLibrary, gtk_source_completion_words_library, G_TYPE_OBJECT)
 
 static void
 gtk_source_completion_words_library_finalize (GObject *object)
@@ -63,22 +64,30 @@ gtk_source_completion_words_library_class_init (GtkSourceCompletionWordsLibraryC
 		              G_TYPE_FROM_CLASS (klass),
 		              G_SIGNAL_RUN_LAST,
 		              0,
-		              NULL, NULL, NULL,
-		              G_TYPE_NONE, 0);
+		              NULL,
+		              NULL,
+		              g_cclosure_marshal_VOID__VOID,
+		              G_TYPE_NONE,
+		              0);
 
 	signals[UNLOCK] =
 		g_signal_new ("unlock",
 		              G_TYPE_FROM_CLASS (klass),
 		              G_SIGNAL_RUN_LAST,
 		              0,
-		              NULL, NULL, NULL,
-		              G_TYPE_NONE, 0);
+		              NULL,
+		              NULL,
+		              g_cclosure_marshal_VOID__VOID,
+		              G_TYPE_NONE,
+		              0);
+
+	g_type_class_add_private (object_class, sizeof(GtkSourceCompletionWordsLibraryPrivate));
 }
 
 static void
 gtk_source_completion_words_library_init (GtkSourceCompletionWordsLibrary *self)
 {
-	self->priv = gtk_source_completion_words_library_get_instance_private (self);
+	self->priv = GTK_SOURCE_COMPLETION_WORDS_LIBRARY_GET_PRIVATE (self);
 
 	self->priv->store = g_sequence_new ((GDestroyNotify)g_object_unref);
 }
@@ -120,17 +129,12 @@ iter_match_prefix (GSequenceIter *iter,
                    gint           len)
 {
 	GtkSourceCompletionWordsProposal *item;
-	const gchar *proposal_word;
 
 	item = gtk_source_completion_words_library_get_proposal (iter);
-	proposal_word = gtk_source_completion_words_proposal_get_word (item);
 
-	if (len == -1)
-	{
-		len = strlen (word);
-	}
-
-	return strncmp (proposal_word, word, len) == 0;
+	return strncmp (gtk_source_completion_words_proposal_get_word (item),
+	                word,
+	                len != -1 ? len : strlen (word)) == 0;
 }
 
 GtkSourceCompletionWordsProposal *
