@@ -76,13 +76,13 @@
 
 #include "gtksourcecompletion.h"
 #include "gtksourcecompletion-private.h"
-#include "gtksourcecompletionutils.h"
 #include "gtksourcecompletionmodel.h"
 #include "gtksourcecompletioncontext.h"
 #include "gtksourcecompletioninfo.h"
 #include "gtksourcecompletionproposal.h"
 #include "gtksourcecompletionprovider.h"
 #include "gtksourcebuffer.h"
+#include "gtksourceview.h"
 #include "gtksourceview-marshal.h"
 #include "gtksourceview-i18n.h"
 
@@ -131,9 +131,13 @@ enum
 
 struct _GtkSourceCompletionPrivate
 {
-	GtkWindow *main_window;
+	GtkSourceCompletionInfo *main_window;
 	GtkSourceCompletionInfo *info_window;
 
+	/* Bottom bar, containing the "Details" button and the selection image
+	 * and label. */
+	GtkWidget *bottom_bar;
+
 	/* Image and label in the bottom bar, on the right, for showing which
 	 * provider(s) are selected. */
 	GtkImage *selection_image;
@@ -192,31 +196,27 @@ scroll_to_iter (GtkSourceCompletion *completion,
 	GtkTreePath *path;
 	GtkTreeIter prev_iter = *iter;
 
-	path = gtk_tree_model_get_path (GTK_TREE_MODEL (completion->priv->model_proposals),
-					iter);
-
-	gtk_tree_view_scroll_to_cell (completion->priv->tree_view_proposals,
-				      path, NULL,
-				      FALSE, 0, 0);
-	gtk_tree_path_free (path);
-
+	/* If we want to scroll to the first proposal of a provider, it's better
+	 * to show the header too, if there is a header. */
 	if (gtk_source_completion_model_iter_previous (completion->priv->model_proposals, &prev_iter) &&
 	    gtk_source_completion_model_iter_is_header (completion->priv->model_proposals, &prev_iter))
 	{
-		/* If we want to scroll to the first proposal of a provider,
-		 * it's better to show the header too, if there is a header.
-		 * We first scroll to the proposal, and then to the
-		 * header, so we are sure that the proposal is visible.
-		 */
-
 		path = gtk_tree_model_get_path (GTK_TREE_MODEL (completion->priv->model_proposals),
 						&prev_iter);
-
-		gtk_tree_view_scroll_to_cell (completion->priv->tree_view_proposals,
-					      path, NULL,
-					      FALSE, 0, 0);
-		gtk_tree_path_free (path);
 	}
+	else
+	{
+		path = gtk_tree_model_get_path (GTK_TREE_MODEL (completion->priv->model_proposals),
+						iter);
+	}
+
+	gtk_tree_view_scroll_to_cell (completion->priv->tree_view_proposals,
+				      path,
+				      NULL,
+				      FALSE,
+				      0,
+				      0);
+	gtk_tree_path_free (path);
 }
 
 /* Returns %TRUE if a proposal is selected.
@@ -365,6 +365,66 @@ reset_completion (GtkSourceCompletion *completion)
 	completion->priv->active_providers = NULL;
 }
 
+/* A separator is a character like (, a space etc. An _ is not a separator. */
+static gboolean
+is_separator (const gunichar ch)
+{
+	if (g_unichar_isprint (ch) &&
+	    (g_unichar_isalnum (ch) || ch == g_utf8_get_char ("_")))
+	{
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+/* Assigns @start_word to the start position of the word, and @end_word to the
+ * end position.
+ */
+static void
+get_word_iter (GtkTextBuffer *buffer,
+	       GtkTextIter   *start_word,
+	       GtkTextIter   *end_word)
+{
+	gtk_text_buffer_get_iter_at_mark (buffer,
+	                                  end_word,
+	                                  gtk_text_buffer_get_insert (buffer));
+
+	*start_word = *end_word;
+
+	while (gtk_text_iter_backward_char (start_word))
+	{
+		gunichar ch = gtk_text_iter_get_char (start_word);
+
+		if (is_separator (ch))
+		{
+			gtk_text_iter_forward_char (start_word);
+			return;
+		}
+	}
+}
+
+static void
+replace_current_word (GtkTextBuffer *buffer,
+		      const gchar   *new_text)
+{
+	GtkTextIter word_start;
+	GtkTextIter word_end;
+
+	get_word_iter (buffer, &word_start, &word_end);
+
+	gtk_text_buffer_begin_user_action (buffer);
+
+	gtk_text_buffer_delete (buffer, &word_start, &word_end);
+
+	if (new_text != NULL)
+	{
+		gtk_text_buffer_insert (buffer, &word_start, new_text, -1);
+	}
+
+	gtk_text_buffer_end_user_action (buffer);
+}
+
 static void
 update_window_position (GtkSourceCompletion *completion)
 {
@@ -392,12 +452,12 @@ update_window_position (GtkSourceCompletion *completion)
 		GtkTextIter end_word;
 		GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (completion->priv->view));
 
-		gtk_source_completion_utils_get_word_iter (buffer, &iter, &end_word);
+		get_word_iter (buffer, &iter, &end_word);
 	}
 
-	gtk_source_completion_utils_move_to_iter (completion->priv->main_window,
-	                                          completion->priv->view,
-	                                          &iter);
+	gtk_source_completion_info_move_to_iter (completion->priv->main_window,
+	                                         GTK_TEXT_VIEW (completion->priv->view),
+	                                         &iter);
 }
 
 static void
@@ -411,23 +471,104 @@ update_tree_view_visibility (GtkSourceCompletion *completion)
 	              NULL);
 }
 
+static void
+set_info_widget (GtkSourceCompletion *completion,
+		 GtkWidget           *new_widget)
+{
+	GtkWidget *cur_widget = gtk_bin_get_child (GTK_BIN (completion->priv->info_window));
+
+	if (cur_widget == new_widget)
+	{
+		return;
+	}
+
+	if (cur_widget != NULL)
+	{
+		gtk_container_remove (GTK_CONTAINER (completion->priv->info_window), cur_widget);
+	}
+
+	gtk_container_add (GTK_CONTAINER (completion->priv->info_window), new_widget);
+}
+
+static void
+update_proposal_info_state (GtkSourceCompletion *completion)
+{
+	GtkSourceCompletionProvider *provider = NULL;
+	GtkSourceCompletionProposal *proposal = NULL;
+	GtkWidget *info_widget;
+
+	if (!get_selected_proposal (completion, &provider, &proposal))
+	{
+		gtk_widget_set_sensitive (GTK_WIDGET (completion->priv->info_button), FALSE);
+		return;
+	}
+
+	info_widget = gtk_source_completion_provider_get_info_widget (provider, proposal);
+
+	if (info_widget != NULL)
+	{
+		set_info_widget (completion, info_widget);
+		gtk_widget_set_sensitive (GTK_WIDGET (completion->priv->info_button), TRUE);
+
+		gtk_source_completion_provider_update_info (provider,
+			                                    proposal,
+			                                    completion->priv->info_window);
+	}
+	else
+	{
+		gchar *text = gtk_source_completion_proposal_get_info (proposal);
+
+		if (text != NULL)
+		{
+			set_info_widget (completion, GTK_WIDGET (completion->priv->default_info));
+			gtk_widget_set_sensitive (GTK_WIDGET (completion->priv->info_button), TRUE);
+
+			gtk_label_set_markup (completion->priv->default_info, text);
+			g_free (text);
+		}
+		else
+		{
+			gtk_widget_set_sensitive (GTK_WIDGET (completion->priv->info_button), FALSE);
+		}
+	}
+
+	g_object_unref (provider);
+	g_object_unref (proposal);
+}
+
+static void
+update_info_window_visibility (GtkSourceCompletion *completion)
+{
+	if (gtk_widget_get_sensitive (GTK_WIDGET (completion->priv->info_button)) &&
+	    gtk_toggle_button_get_active (completion->priv->info_button))
+	{
+		gtk_widget_show (GTK_WIDGET (completion->priv->info_window));
+	}
+	else
+	{
+		gtk_widget_hide (GTK_WIDGET (completion->priv->info_window));
+	}
+}
+
+static void
+update_proposal_info (GtkSourceCompletion *completion)
+{
+	update_proposal_info_state (completion);
+	update_info_window_visibility (completion);
+}
+
 static void
 gtk_source_completion_show_default (GtkSourceCompletion *completion)
 {
 	gtk_widget_show (GTK_WIDGET (completion->priv->main_window));
 
-	if (completion->priv->remember_info_visibility)
-	{
-		if (gtk_toggle_button_get_active (completion->priv->info_button))
-		{
-			gtk_widget_show (GTK_WIDGET (completion->priv->info_window));
-		}
-	}
-	else
+	if (!completion->priv->remember_info_visibility)
 	{
 		gtk_toggle_button_set_active (completion->priv->info_button, FALSE);
 	}
 
+	update_proposal_info (completion);
+
 	gtk_widget_grab_focus (GTK_WIDGET (completion->priv->view));
 }
 
@@ -477,7 +618,7 @@ gtk_source_completion_activate_proposal (GtkSourceCompletion *completion)
 		}
 		else
 		{
-			gtk_source_completion_utils_replace_current_word (buffer, text);
+			replace_current_word (buffer, text);
 		}
 
 		g_free (text);
@@ -491,25 +632,6 @@ gtk_source_completion_activate_proposal (GtkSourceCompletion *completion)
 	g_object_unref (proposal);
 }
 
-static void
-set_info_widget (GtkSourceCompletion *completion,
-		 GtkWidget           *new_widget)
-{
-	GtkWidget *cur_widget = gtk_bin_get_child (GTK_BIN (completion->priv->info_window));
-
-	if (cur_widget == new_widget)
-	{
-		return;
-	}
-
-	if (cur_widget != NULL)
-	{
-		gtk_container_remove (GTK_CONTAINER (completion->priv->info_window), cur_widget);
-	}
-
-	gtk_container_add (GTK_CONTAINER (completion->priv->info_window), new_widget);
-}
-
 static void
 update_info_position (GtkSourceCompletion *completion)
 {
@@ -519,11 +641,11 @@ update_info_position (GtkSourceCompletion *completion)
 	gint screen_width;
 	gint info_width;
 
-	gtk_window_get_position (completion->priv->main_window, &x, &y);
-	gtk_window_get_size (completion->priv->main_window, &width, &height);
+	gtk_window_get_position (GTK_WINDOW (completion->priv->main_window), &x, &y);
+	gtk_window_get_size (GTK_WINDOW (completion->priv->main_window), &width, &height);
 	gtk_window_get_size (GTK_WINDOW (completion->priv->info_window), &info_width, NULL);
 
-	screen = gtk_window_get_screen (completion->priv->main_window);
+	screen = gtk_window_get_screen (GTK_WINDOW (completion->priv->main_window));
 	screen_width = gdk_screen_get_width (screen);
 
 	/* Determine on which side to place it */
@@ -539,57 +661,6 @@ update_info_position (GtkSourceCompletion *completion)
 	gtk_window_move (GTK_WINDOW (completion->priv->info_window), x, y);
 }
 
-static void
-update_proposal_info (GtkSourceCompletion *completion)
-{
-	GtkSourceCompletionProvider *provider = NULL;
-	GtkSourceCompletionProposal *proposal = NULL;
-	GtkWidget *info_widget;
-
-	if (!get_selected_proposal (completion, &provider, &proposal))
-	{
-		set_info_widget (completion, GTK_WIDGET (completion->priv->default_info));
-
-		gtk_label_set_markup (completion->priv->default_info,
-				      _("No extra information available"));
-
-		return;
-	}
-
-	info_widget = gtk_source_completion_provider_get_info_widget (provider, proposal);
-
-	if (info_widget != NULL)
-	{
-		set_info_widget (completion, info_widget);
-
-		gtk_source_completion_provider_update_info (provider,
-			                                    proposal,
-			                                    completion->priv->info_window);
-	}
-	else
-	{
-		gchar *text;
-
-		set_info_widget (completion, GTK_WIDGET (completion->priv->default_info));
-
-		text = gtk_source_completion_proposal_get_info (proposal);
-
-		if (text != NULL)
-		{
-			gtk_label_set_markup (completion->priv->default_info, text);
-			g_free (text);
-		}
-		else
-		{
-			gtk_label_set_markup (completion->priv->default_info,
-					      _("No extra information available"));
-		}
-	}
-
-	g_object_unref (provider);
-	g_object_unref (proposal);
-}
-
 static GtkSourceCompletionProvider *
 get_visible_provider (GtkSourceCompletion *completion)
 {
@@ -957,7 +1028,6 @@ gtk_source_completion_move_page (GtkSourceCompletion *completion,
 	}
 
 	gtk_tree_view_set_model (completion->priv->tree_view_proposals, NULL);
-	gtk_tree_view_columns_autosize (completion->priv->tree_view_proposals);
 
 	gtk_source_completion_model_set_visible_providers (completion->priv->model_proposals,
 							   visible_providers);
@@ -1125,7 +1195,8 @@ view_key_press_event_cb (GtkSourceView       *view,
 
 	/* Handle info button mnemonic */
 	if ((mod & GDK_MOD1_MASK) != 0 &&
-	    event->keyval == mnemonic_keyval)
+	    event->keyval == mnemonic_keyval &&
+	    gtk_widget_get_sensitive (GTK_WIDGET (completion->priv->info_button)))
 	{
 		gtk_toggle_button_set_active (completion->priv->info_button,
 		                              !gtk_toggle_button_get_active (completion->priv->info_button));
@@ -1172,7 +1243,7 @@ update_transient_for_info (GObject             *window,
                            GtkSourceCompletion *completion)
 {
 	gtk_window_set_transient_for (GTK_WINDOW (completion->priv->info_window),
-				      gtk_window_get_transient_for (completion->priv->main_window));
+				      gtk_window_get_transient_for (GTK_WINDOW (completion->priv->main_window)));
 }
 
 static void
@@ -1210,8 +1281,6 @@ update_completion (GtkSourceCompletion        *completion,
 
 	/* Create a new CompletionModel */
 	gtk_tree_view_set_model (completion->priv->tree_view_proposals, NULL);
-	gtk_tree_view_columns_autosize (completion->priv->tree_view_proposals);
-
 	replace_model (completion);
 
 	for (item = providers_copy; item != NULL; item = g_list_next (item))
@@ -1356,6 +1425,32 @@ buffer_insert_text_cb (GtkTextBuffer       *buffer,
 	}
 }
 
+static void
+update_bottom_bar_visibility (GtkSourceCompletion *completion)
+{
+	GList *providers;
+	guint nb_providers;
+
+	providers = gtk_source_completion_model_get_providers (completion->priv->model_proposals);
+	nb_providers = g_list_length (providers);
+	g_list_free (providers);
+
+	if (nb_providers > 1)
+	{
+		gtk_widget_show (completion->priv->bottom_bar);
+		return;
+	}
+
+	if (gtk_source_completion_model_has_info (completion->priv->model_proposals))
+	{
+		gtk_widget_show (completion->priv->bottom_bar);
+	}
+	else
+	{
+		gtk_widget_hide (completion->priv->bottom_bar);
+	}
+}
+
 static void
 populating_done (GtkSourceCompletion        *completion,
                  GtkSourceCompletionContext *context)
@@ -1370,16 +1465,21 @@ populating_done (GtkSourceCompletion        *completion,
 				 GTK_TREE_MODEL (completion->priv->model_proposals));
 
 	update_selection_label (completion);
-
-	if (!check_first_selected (completion))
-	{
-		update_window_position (completion);
-	}
+	update_bottom_bar_visibility (completion);
 
 	if (!gtk_widget_get_visible (GTK_WIDGET (completion->priv->main_window)))
 	{
 		g_signal_emit (completion, signals[SHOW], 0);
 	}
+
+	if (!check_first_selected (completion))
+	{
+		/* Update the window position only if the first proposal is not
+		 * selected, because if it is selected, the window position will
+		 * already be updated.
+		 */
+		update_window_position (completion);
+	}
 }
 
 static void
@@ -2141,22 +2241,23 @@ static void
 init_main_window (GtkSourceCompletion *completion,
 		  GtkBuilder          *builder)
 {
-	completion->priv->main_window = GTK_WINDOW (gtk_builder_get_object (builder, "main_window"));
+	completion->priv->main_window = GTK_SOURCE_COMPLETION_INFO (gtk_builder_get_object (builder, "main_window"));
 	completion->priv->info_button = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "info_button"));
 	completion->priv->selection_image = GTK_IMAGE (gtk_builder_get_object (builder, "selection_image"));
 	completion->priv->selection_label = GTK_LABEL (gtk_builder_get_object (builder, "selection_label"));
+	completion->priv->bottom_bar = GTK_WIDGET (gtk_builder_get_object (builder, "bottom_bar"));
 
-	gtk_window_set_attached_to (completion->priv->main_window,
+	gtk_window_set_attached_to (GTK_WINDOW (completion->priv->main_window),
 				    GTK_WIDGET (completion->priv->view));
 
 	gtk_widget_set_size_request (GTK_WIDGET (completion->priv->main_window),
 	                             WINDOW_WIDTH,
 	                             WINDOW_HEIGHT);
 
-	g_signal_connect (completion->priv->main_window,
-			  "configure-event",
-			  G_CALLBACK (gtk_source_completion_configure_event),
-			  completion);
+	g_signal_connect_after (completion->priv->main_window,
+				"configure-event",
+				G_CALLBACK (gtk_source_completion_configure_event),
+				completion);
 
 	g_signal_connect (completion->priv->main_window,
 			  "delete-event",
@@ -2167,6 +2268,11 @@ init_main_window (GtkSourceCompletion *completion,
 	                  "notify::transient-for",
 	                  G_CALLBACK (update_transient_for_info),
 	                  completion);
+
+	g_signal_connect_swapped (completion->priv->info_button,
+				  "toggled",
+				  G_CALLBACK (update_info_window_visibility),
+				  completion);
 }
 
 static void
@@ -2178,10 +2284,6 @@ init_info_window (GtkSourceCompletion *completion)
 	gtk_window_set_attached_to (GTK_WINDOW (completion->priv->info_window),
 				    GTK_WIDGET (completion->priv->main_window));
 
-	g_object_bind_property (completion->priv->info_button, "active",
-				completion->priv->info_window, "visible",
-				G_BINDING_DEFAULT);
-
 	g_signal_connect_swapped (completion->priv->info_window,
 				  "size-allocate",
 				  G_CALLBACK (update_info_position),
@@ -2564,9 +2666,9 @@ gtk_source_completion_move_window (GtkSourceCompletion *completion,
 		return;
 	}
 
-	gtk_source_completion_utils_move_to_iter (completion->priv->main_window,
-	                                          completion->priv->view,
-	                                          iter);
+	gtk_source_completion_info_move_to_iter (completion->priv->main_window,
+	                                         GTK_TEXT_VIEW (completion->priv->view),
+	                                         iter);
 }
 
 /**
