@@ -19,9 +19,17 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+/**
+ * SECTION:completionitem
+ * @title: GtkSourceCompletionItem
+ * @short_description: Simple implementation of #GtkSourceCompletionProposal
+ *
+ * The #GtkSourceCompletionItem class is a simple implementation of the
+ * #GtkSourceCompletionProposal interface.
+ */
+
 #include "gtksourcecompletionitem.h"
 #include "gtksourcecompletionproposal.h"
-#include "gtksourcecompletionutils.h"
 #include "gtksourceview-i18n.h"
 
 #define GTK_SOURCE_COMPLETION_ITEM_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_SOURCE_TYPE_COMPLETION_ITEM, GtkSourceCompletionItemPrivate))
