@@ -145,7 +145,7 @@ void
 gtk_source_pixbuf_helper_set_icon_name (GtkSourcePixbufHelper *helper,
                                         const gchar           *icon_name)
 {
-	helper->type = ICON_TYPE_NAME;
+	helper->type = ICON_TYPE_STOCK;
 
 	if (helper->icon_name)
 	{
@@ -216,8 +216,6 @@ from_pixbuf (GtkSourcePixbufHelper *helper,
 	                                            GDK_INTERP_BILINEAR));
 }
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
-
 static void
 from_stock (GtkSourcePixbufHelper *helper,
             GtkWidget             *widget,
@@ -241,9 +239,6 @@ from_stock (GtkSourcePixbufHelper *helper,
 	                                                  helper->stock_id,
 	                                                  icon_size));
 }
-
-G_GNUC_END_IGNORE_DEPRECATIONS;
-
 static void
 from_gicon (GtkSourcePixbufHelper *helper,
             GtkWidget             *widget,
@@ -279,37 +274,20 @@ from_name (GtkSourcePixbufHelper *helper,
 	GtkIconTheme *icon_theme;
 	GtkIconInfo *info;
 	GtkIconLookupFlags flags;
-	gint scale;
 
 	screen = gtk_widget_get_screen (widget);
 	icon_theme = gtk_icon_theme_get_for_screen (screen);
 
 	flags = GTK_ICON_LOOKUP_USE_BUILTIN;
-        scale = gtk_widget_get_scale_factor (widget);
 
-	info = gtk_icon_theme_lookup_icon_for_scale (icon_theme,
-	                                             helper->icon_name,
-	                                             size,
-	                                             scale,
-	                                             flags);
+	info = gtk_icon_theme_lookup_icon (icon_theme,
+	                                   helper->icon_name,
+	                                   size,
+	                                   flags);
 
 	if (info)
 	{
-		GdkPixbuf *pixbuf;
-
-		if (gtk_icon_info_is_symbolic (info))
-		{
-			GtkStyleContext *context;
-
-			context = gtk_widget_get_style_context (widget);
-			pixbuf = gtk_icon_info_load_symbolic_for_context (info, context, NULL, NULL);
-		}
-		else
-		{
-			pixbuf = gtk_icon_info_load_icon (info, NULL);
-		}
-
-		set_cache (helper, pixbuf);
+		set_cache (helper, gtk_icon_info_load_icon (info, NULL));
 	}
 }
 
@@ -338,8 +316,6 @@ gtk_source_pixbuf_helper_render (GtkSourcePixbufHelper *helper,
 		case ICON_TYPE_NAME:
 			from_name (helper, widget, size);
 			break;
-		default:
-			g_assert_not_reached ();
 	}
 
 	return helper->cached_pixbuf;
